<?php

/**
 *
 * Callback for /admin/config/media/orbit.
 */
function bynder_orbit_form($form, &$form_state)
{
    $form['bynder_orbit_settings'] = array(
        '#type' => 'fieldset',
        '#title' => t('Bynder Orbit OAuth settings'),
        '#description' => t('You will need to enable the Drupal 
          integration in your Bynder Orbit account in order to generate your 
          OAuth client ID and secret. <br>To do so, access the Drupal 7 section of 
          the <strong>Bynder Orbit Marketplace</strong> in <strong>Account overview</strong>.'),
        '#collapsible' => true
    );

    $form['bynder_orbit_settings']['oauth_client_id'] = array(
        '#type' => 'textfield',
        '#title' => t('Client ID'),
        '#default_value' => variable_get('oauth_client_id', ''),
        '#maxlength' => 500,
        '#description' => t('Bynder Orbit OAuth client ID.'),
        '#required' => true
    );

    $form['bynder_orbit_settings']['oauth_client_secret'] = array(
      '#type' => 'textfield',
      '#title' => t('Client secret'),
      '#default_value' => variable_get('oauth_client_secret', ''),
      '#maxlength' => 500,
      '#description' => t('Bynder Orbit OAuth client secret.'),
      '#required' => true
    );

    $form['#submit'][] = 'bynder_orbit_form_submit';

    return system_settings_form($form);
}

/**
 * Admin form submit, force reloading previews the first time we save the form.
 */
function bynder_orbit_form_submit($form, &$form_state)
{
  bynder_orbit_batch_reload_previews($form, $form_state);
  cache_clear_all('*', 'cache_bynder_orbit', true);
}

/**
 * Batch callback to fetch preview information.
 */
function bynder_orbit_batch_reload_previews($form, &$form_state)
{
    $batch = array(
        'operations' => array(
          array('bynder_orbit_batch_reload_previews_process', array()),
        ),
        'finished' => 'bynder_orbit_batch_reload_previews_finished',
        'title' => t('Batch to reload Bynder previews information'),
        'init_message' => t('Bynder previews batch is starting...'),
        'progress_message' => t('Processed @current out of @total.'),
        'error_message' => t('Fix has encountered an error.')
    );
    batch_set($batch);
}

/**
 * Processes the preview list and sets new image styles for each one.
 */
function bynder_orbit_batch_reload_previews_process(&$context)
{
    $context['message'] = "Updating previews data.";
    $previews = [
      'small',
      'thumbnail',
      'overview',
      'detail'
    ];
    variable_set('orbit_previews', serialize($previews));
    foreach ($previews as $preview) {
        try {
            if (!image_style_load('bynder_orbit_' . $preview)) {
                $context['message'] = "Creating image style for preview: $preview";
                image_style_save(
                    array(
                        'name' => 'bynder_orbit_' . $preview,
                        'label' => t('Bynder Orbit preview: ' . $preview)
                    )
                );

            }
        } catch (Exception $e) {
            watchdog('bynder', $e->getMessage());
            $context['finished'] = 1;
            $context['results']['error'] = t("There was an error while saving new image styles.");
        }
    }
}

/**
 * Batch finished callback.
 */
function bynder_orbit_batch_reload_previews_finished($success, $results, $operations)
{
    if ($success) {
      if(isset($results['error'])) {
        drupal_set_message($results['error'], 'error');
      }
      else {
        drupal_set_message('Finished setting up image styles!');
      }
    } else {
        $error_operation = reset($operations);
        $message = t('An error occurred while processing %error_operation with arguments: @arguments', array(
            '%error_operation' => $error_operation[0],
            '@arguments' => print_r($error_operation[1], true)
        ));
        drupal_set_message($message, 'error');
    }
}

