<?php

/**
 * @file
 * Manages the Bynder media browser functionality
 */

const BYNDER_SEARCH_PAGE_SIZE = 10;

/**
 * Create a form to add files from Orbit.
 */
function bynder_orbit_media_browser($form, &$form_state = array(), $redirect = null)
{
  $orbitApi = OrbitMediaApi::createOrbitClient();
  $tokens = $orbitApi->hasAccessToken();
  if (!$orbitApi->hasSettings()) {
    $form['message'] = [
      '#markup' => t('You haven\'t set up the <a href="' . url('admin/config/media/orbit',
          array('absolute' => true)) . '" class="oauth-link" target="_blank">Bynder Orbit settings</a> properly, please do so before importing assets.'),
      '#prefix' => '<p>',
      '#suffix' => '</p>',
    ];
  } elseif (!$tokens || isset($tokens['code_verifier'])) {
    return bynder_orbit_media_browser_get_login();
  } else {

    $bynder_orbit_media_browser = bynder_orbit_generate_search($form_state);

    if(isset($bynder_orbit_media_browser['error']) && !$tokens) {
      return bynder_orbit_media_browser_get_login();
    }
    else {
      unset($bynder_orbit_media_browser['error']);
      $form['bynder_orbit_media_browser'] = $bynder_orbit_media_browser;
    }

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Submit',
    );

    $form['#attributes'] = array('class' => array('bynder-orbit-search-form'));

    $form['#attached']['library'][] = array('bynder_orbit', 'bynder_orbit_media');
    $form['#attached']['css'][] = array(
      'type' => 'external',
      'data' => '//maxcdn.bootstrapcdn.com/font-awesome/4.1.0/css/font-awesome.css',
    );
  }
    return $form;
}


function bynder_orbit_media_browser_get_login() {
  $form['message'] = [
    '#markup' => t('You need to <a href="' . url('orbit-oauth',
        array('absolute' => true)) . '" class="oauth-link" target="_blank">log in to Orbit</a> before importing assets.'),
    '#prefix' => '<p>',
    '#suffix' => '</p>',
  ];
  $form['reload'] = [
    '#type' => 'button',
    '#value' => 'Reload after submit',
    '#attached' => ['library' => ['orbit/oauth']],
    '#attributes' => ['class' => ['oauth-reload', 'visually-hidden']],
  ];
  return $form;
}


function bynder_orbit_oauth_login()
{
  if (isset($_GET['state'])&& isset($_GET['code'])) {
    $finishTokenRetrieval = OrbitMediaApi::createOrbitClient()->finishOAuthTokenRetrieval($_GET['code'], $_GET['state']);
    if(isset($finishTokenRetrieval['error'])) {
      $_SESSION['bynderLoginError'] = $finishTokenRetrieval['error'];
    }
    return [
      '#markup' => '<script>window.close()</script>',
      '#allowed_tags' => ['script'],
    ];
  } elseif (isset($_GET['oauth_check_login'])) {
    $accessToken = OrbitMediaApi::createOrbitClient()->hasAccessToken();
    if ($accessToken && isset($accessToken['oauth_access_token'])) {
      drupal_json_output(true);
    } elseif (isset($_SESSION['bynderLoginError'])) {
      unset($_SESSION['bynderLoginError']);
      drupal_set_message(t('There is something wrong with your Bynder configuration, please confirm the details 
                in @settings and try again',
        array('@settings' => url('admin/config/media/orbit', array('absolute' => true)))), 'error', FALSE);
      drupal_json_output(true);
    } else {
      drupal_json_output(false);
    }
  } else {
    try {
      OrbitMediaApi::createOrbitClient()->initiateOAuthTokenRetrieval();
    } catch (Exception $e) {
      $_SESSION['bynderLoginError'] = true;
      return [
        '#markup' => '<script>window.close()</script>',
        '#allowed_tags' => ['script'],
      ];
    }
  }
}

/**
 * Generate search page.
 */
function bynder_orbit_generate_search($form_state)
{
  $search = bynder_orbit_get_media_form_filter_value($form_state, 'search_field', '');
    // Create search section
    $data['search'] = array(
        '#type' => 'fieldset',
        '#collapsible' => false,
        '#collapsed' => false,
        '#attributes' => array(
            'class' => array(
                'bynder-search-form',
            ),
        ),
    );
    $data['search']['search_field'] = array(
      '#title' => t('Search'),
      '#type' => 'textfield',
        '#default_value' => isset($search) ? $search : '',
        '#description' => "<p class='alert alert-warning' style='display:none'><span class='text'></span></p>",
    );

  $sort_order = bynder_orbit_get_media_form_filter_value($form_state, 'sort_order', '-created_at');

    $data['search']['sort_order'] = array(
      '#title' => t('Sort By'),
      '#type' => 'select',
      '#options' => array(
        'name' => t('Name (A-Z)'),
        '-name' => t('Name (Z-A)'),
        '-created_at' => t('Date added (Newest - Oldest)'),
        'created_at' => t('Date added (Oldest - Newest)')
      ),
      '#default_value' => $sort_order
    );

  $extension_filter = bynder_orbit_get_media_form_filter_value($form_state, 'filter_extension', '');

  $data['search']['filter_extension'] = array(
      '#title' => t('Extension'),
      '#type' => 'select',
      '#multiple' => true,
      '#options' => array(
        'jpg' => t('JPG'),
        'jpeg' => t('JPEG'),
        'png' => t('PNG'),
        'svg' => t('SVG')
      ),
      '#default_value' => isset($extension_filter) ? array_values(explode(',', $extension_filter)) : ''
    );

  $data['search']['button'] = array(
        '#type' => 'button',
        '#value' => t('Search'),
    );

    $data['results'] = array(
        '#type' => 'container',
        '#attributes' => array(
            'id' => array('results_list'),
            'class' => array('grid'),
        ),
    );
    $results = bynder_orbit_search_results($search, $sort_order, $extension_filter, BYNDER_SEARCH_PAGE_SIZE);

    if (isset($results['data']) && !isset($results['error'])) {
        // Create the results section
        $data['results'][] = bynder_orbit_generate_results_grid($results['data']);
        if (isset($results['totalNrOfResults'])) {
            $data['pager_container'] = array(
                '#type' => 'container',
                '#attributes' => array(
                    'class' => array('.js-form-type-entity-browser-pager'),
                ),
            );
            $data['pager_container']['total_results'] = array(
                '#type' => 'html_tag',
                '#tag' => 'h3',
                '#attributes' => array(
                    'class' => 'total_results'
                ),
                '#value' => t('Total Results: ') . $results['totalNrOfResults']
            );
            $data['pager_container']['pager'] = bynder_generate_pager(
                $results['totalNrOfResults'], 0, array('search_field' => $search)
            );
        }
    } else {
        if (isset($results['error'])) {
            drupal_set_message(t($results['error']), 'error', FALSE);
          $data['error'] = $results['error'];
        }
        $data['results'][] = bynder_orbit_no_results_found();
    }

    return $data;
}

function bynder_orbit_search_results($search, $sort_order, $filters = '', $nr_of_results = 5)
{
    $start_index = 0;
    $query = \Orbit\Api\Queries\AssetListQuery::create();

    if (isset($search) && strlen($search) > 0) {
        $query->setQuery(urlencode($search));
    }

    if(isset($sort_order)) {
      $query->setSort($sort_order);
    }

    if(!empty($filters)) {
      $extension_filter = $filters;
    }

  if(isset($_POST['search_field']) && $_POST['search_field'] != $search ||
    isset($_POST['filter_extension']) && $_POST['filter_extension'] != $filters) {
    $_GET['page'] = 0; // If we're doing a search, make sure we get the first page.
  }

    if (isset($_GET['page'])) {
        $current_page = (int)check_plain($_GET['page']);
        $start_index = $current_page * $nr_of_results;
    }
    $query
        ->setLimit($nr_of_results)
        ->setFrom($start_index)
        ->setExtension(!empty($extension_filter) ? $extension_filter : 'jpeg,jpg,png,svg'); // Can only use images.

    $result = OrbitMediaApi::createOrbitClient()->getAssets($query);
    if (!empty($result) && isset($result['paging']['count'])) {
        $result['totalNrOfResults'] = $result['paging']['count'];
    }
    return $result;
}

/**
 * Organize results from search engine to visualize them.
 */
function bynder_orbit_generate_results_grid($results)
{
    $output = [];
    foreach ($results as $key => $media_item) {
        if (isset($media_item['current_version']['previews']['images']['thumbnail'])) {
            $output['thumbnail-' . $media_item['asset_id']] = array(
                '#type' => 'container',
                '#attributes' => array('id' => $media_item['asset_id'], 'class' => array('grid-item')),
            );
            $output['thumbnail-' . $media_item['asset_id']]['check_' . $media_item['asset_id']] = array(
                '#type' => 'checkbox',
                '#parents' => array('selected_assets', $media_item['asset_id']),
                '#attributes' => array('class' => ['item-selector']),
            );
            $output['thumbnail-' . $media_item['asset_id']]['image'] = array(
                '#name' => $media_item['name'],
                '#type' => 'markup',
                '#markup' => '<img src="' . $media_item['current_version']['previews']['images']['thumbnail']['url'] . '" alt="' . $media_item['name'] . '"/>
                                <div class="more-info">
                                  <h4 class="name">' . $media_item['name'] . '</h4>
                                </div>'
            );
        }
    }
    return $output;
}

function bynder_orbit_media_browser_submit($form, &$form_state)
{
    $selected_assets = isset($form_state['values']['selected_assets']) ? $form_state['values']['selected_assets'] : null;
    if ($selected_assets) {
        $fids = array();
        foreach ($selected_assets as $assetId => $selected) {
            if ($selected) {
                $result = bynder_orbit_save_image($assetId, $form);
                if (isset($result['success'])) {
                    $fids[] = $result['fid'];
                } else {
                    drupal_set_message(t($result['message']), 'error');

                }
            }
        }
        // Save the files
        if (!empty($fids)) {
            if (current_path() != 'admin/content/media/add/orbit') {
                drupal_goto('/media/browser', array(
                    'query' => array(
                        'render' => 'media-popup',
                        'fid' => $fids
                    )
                ));
            } else {
                drupal_set_message(t("Successfully added the selected assets."), 'status');
                drupal_goto('admin/content/file');
            }
        }
        else {
          drupal_set_message(t("Couldn't add any assets, please check the error logs for more information."), 'warning');
        }
    }
}

function bynder_orbit_save_image($id, $form = null)
{
    $selected_media = OrbitMediaApi::createOrbitClient()->getAssetById(
      \Orbit\Api\Queries\AssetViewQuery::create($id)
    );

    if (isset($selected_media['error'])) {
      return false;
    }

    $idHash = $selected_media['name'];

    $uri = file_stream_wrapper_uri_normalize("orbit://f/$id/i/$idHash");

    $files = file_load_multiple(array(), array('uri' => $uri));
    $file = !empty($files) ? reset($files) : file_uri_to_object($uri);

    if ($file === false) {
      return array('success' => 0, 'message' => 'An error occurred and no file was added to your Library.');
    } elseif (!isset($file->fid)) {
      $file->filemime = 'image/bynder_orbit';
      $file->filesize = 0;
      $file->filename = $selected_media['name'];

      $file = file_save($file);
      field_attach_submit('media', $file, $form, $form_state);
      $file->file = array();
      $file->file[LANGUAGE_NONE] = array();
      $file->file[LANGUAGE_NONE][0] = (array)$file + array(
          'display' => true
        );
      $file->is_new = true;
      field_attach_insert('media', $file);

      bynder_create_media_entry($selected_media, $file->fid);

      return array('success' => 1, 'message' => 'Successfully added media to Library', 'fid' => $file->fid);
    } else {
      return array('success' => 1, 'message' => 'Already exists in Library', 'fid' => $file->fid);
    }
}

function bynder_create_media_entry($selected_media, $fid)
{
    // Save Bynder asset entry.
    $bynderId = $selected_media['asset_id'];
    $name = $selected_media['name'];
    $description = isset($selected_media['description']) ? $selected_media['description'] : $selected_media['name'];
    try {
        db_insert('bynder_orbit_media_entity')
            ->fields(array(
                'bynder_orbit_id' => $bynderId,
                'name' => $name,
                'description' => $description,
                'previews' => serialize($selected_media['current_version']['previews']['images']),
                'fid' => $fid
            ))
            ->execute();

    } catch (Exception $e) {
        // Handle exception
        watchdog('bynder', $e->getMessage());
    }
}

function bynder_orbit_get_media_form_filter_value($form_state, $field_name, $default_value) {

  if (isset($form_state['values'][$field_name])) {
    $field_value = is_array($form_state['values'][$field_name]) ? implode(',', $form_state['values'][$field_name]) : $form_state['values'][$field_name];
    return check_plain($field_value);
  } elseif (isset($_GET[$field_name])) {
    return check_plain($_GET[$field_name]);
  }

  return $default_value;
}

/**
 * Generate paging functionality.
 */
function bynder_generate_pager($num_results, $element = 0, $parameters = array())
{
    if (isset($_POST['search_field'])) {
        $_GET['search_field'] = check_plain($_POST['search_field']);
    }
    if (isset($_POST['sort_order'])) {
        $_GET['sort_order'] = check_plain($_POST['sort_order']);
    }
    if (isset($_POST['filter_extension'])) {
        $_GET['filter_extension'] = check_plain(implode(',', $_POST['filter_extension']));
    }
    pager_default_initialize($num_results, BYNDER_SEARCH_PAGE_SIZE, $element);
    return array(
        '#markup' => theme('pager', array(
            'parameters' => $parameters,
            'element' => $element,
        ))
    );
}

/**
 * Returns a simple "No results found" HTML section
 */
function bynder_orbit_no_results_found()
{
    return array(
        '#name' => 'orbit-no-results-section',
        '#attributes' => array('class' => array('orbit-no-results-section')),
        '#type' => 'markup',
        '#markup' => '<div><h3>' . t('Could not retrieve search results.') . '</h3></div>'
    );
}